//number of bytes in buffer
//#define BSIZE	(4096*2)  
//number of short int per channel N = BIZE/channels/2
//#define NUMBUFFERS	(8)
#include "buffer.h"
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h> //non blocking kbd io

// openal
#include <AL/al.h>
#include <AL/alc.h>

//wave file vars
#include "wave.h"
WAVE_Struct	wave;
FILE		*fp;

FILE * Openwavefile( const char *, WAVE_Struct *, int );
void Prebuffer(void);
void IdleFiltering (void);
void srealfft(int, float *);


//filter
float H[2][BSIZE],X[2][BSIZE],SAVE[2][BSIZE];

// process vars
const char      file[] = {"music.wav"};
ALbyte          DATA   [BSIZE];
ALuint          Buffers[NUMBUFFERS];
ALuint          BufferID;
ALuint          DataSize;
ALuint          DataToRead;
ALint           processed;
ALboolean       bFinished = AL_FALSE;
ALuint          Format;
ALint           State;
ALuint 		Source;

ALuint	buffersreturned=0;
ALuint	buffersinqueue=NUMBUFFERS;
ALboolean bFinishedPlaying;
int N;

//++++++++++++++++++++++++++++++++++++++++++

ALfloat SourcePos[] = { 0.0, 0.0, 0.0 };
ALfloat SourceVel[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerPos[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerVel[] = { 0.0, 0.0, 0.0 };
ALfloat ListenerOri[] = { 0.0, 0.0, -1.0,  0.0, 1.0, 0.0 };




int main(int argc, char *argv[])
{
extern FILE *fp;
extern ALuint  Buffers[NUMBUFFERS];
extern float H[2][BSIZE],X[2][BSIZE],SAVE[2][BSIZE];
extern int N;
int i,j;


//==========================================
//START:OPEN DEFAULT AUDIO DEVICE
 ALCdevice *device;
 ALCcontext *ctx;
 const ALCchar  *name;

  device = alcOpenDevice(NULL);
   if(!device) { printf("cannot open default device\n");exit(0);}

  ctx = alcCreateContext(device,NULL);
    if(ctx == NULL || alcMakeContextCurrent(ctx) == ALC_FALSE)
    {   if(ctx != NULL) alcDestroyContext(ctx);
        alcCloseDevice(device);
        fprintf(stderr, "Could not set a context!\n");
	exit(0); }

    name = NULL;
    if(alcIsExtensionPresent(device, "ALC_ENUMERATE_ALL_EXT"))
        name = alcGetString(device, ALC_ALL_DEVICES_SPECIFIER);
    if(!name || alcGetError(device) != AL_NO_ERROR)
        name = alcGetString(device, ALC_DEVICE_SPECIFIER);
    printf("Opened  \"%s\"\n", name);
//END:OPEN DEFAULT AUDIO DEVICE
//==========================================

//START: GET DATA BUFFERS
	alGetError(); //reset error
	alGenBuffers(NUMBUFFERS,Buffers);
	if(alGetError() != AL_NO_ERROR) printf(" Error alGenBuffers \n");
//END: GET DATA BUFFERS

//open wavefile assumed stereo 16 bit per sample
	fp =Openwavefile( (const char *)&file,&wave, sizeof(WAVE_Struct));

//N = number of (wave.BitsPerSample/8) Byte samples per channel
//Min size needed for signal and filter samples for overlap-save filtering is 2*N

	N = (BSIZE/wave.Channels)/(wave.BitsPerSample/8);

//Begin: INITIALIZE DSP FILTERING
	for(j=0;j<wave.Channels;j++){
	for(i=0;i<BSIZE;i++) {
	    H[j][i]=0;
	    X[j][i]=0;
	    SAVE[j][i]=0;
	}}
//impluse function of the filter
	for(j=0;j<wave.Channels;j++)
	H[j][0]=1;

// DFT of H
	srealfft(N,&H[0][0]);
	srealfft(N,&H[1][0]);

//Prebuffer and filter prebuffer NBUFFERS of data
	Prebuffer();
//End: INITIALIZE DSP FILTERING

// Generate 1 source
	alGetError();
	alGenSources(1,&Source);
	if(alGetError() != AL_NO_ERROR) printf(" location bad 2\n");

//Set source features
	alSourcef (Source, AL_GAIN,     1.0);
	alSourcef (Source, AL_PITCH,    1.0);
	alSourcefv(Source, AL_POSITION, SourcePos);
	alSourcefv(Source, AL_VELOCITY, SourceVel);

//Set listener position, velocity, orientation
	alListenerfv( AL_POSITION,    ListenerPos);
	alListenerfv( AL_VELOCITY,    ListenerVel);
	alListenerfv( AL_ORIENTATION, ListenerOri);

//queue buffers "Source"
	alSourceQueueBuffers(Source,NUMBUFFERS,Buffers);
//Do not loop buffers
	alSourcei(Source,AL_LOOPING,AL_FALSE);

//=============================
// IMPORTANT TO IdleFiltering();
// Initialize buffer status and state
	buffersreturned=0;
	bFinishedPlaying = AL_FALSE;
	buffersinqueue = NUMBUFFERS;
//=============================
//=============================
// SETUP NONBLK LOOP FOR KBD INPUT
// code added using ncurses marked by *
//Loop
//init linux ncurses
	char c;
        initscr();              //*init default screen
        cbreak();               //*process one char at a time
        keypad(stdscr,TRUE);    //*enable function and arrow keys
        nodelay(stdscr,TRUE);   //*no waiting for keyboard input
        noecho();               //*no print to screen when key depressed
//MENU
	mvprintw(0,0,"Controls: \n");   //*print to console window at(0,0) top-upperleft
	printw("================ \n");  //*
	printw("p) Play \n");		//*
	printw("h) Hold \n");		//*
	printw("s) Stop \n");		//*
	printw("q) Quit \n");		//*

//BEGIN: EVENT-LOOP
  c = ' ';                      //*init c
  while( (c=getch()) != 'q')    //BEGIN: EVENT-LOOP
        {

//--------------------------------
 	 IdleFiltering();
//--------------------------------

         if( c !=ERR)           //detected any char input
          {
            switch(c)
              {
                case 'p':
                        {
	  		alGetSourcei(Source,AL_SOURCE_STATE,&State);
	  		if (State != AL_PLAYING) alSourcePlay(Source);
                        mvprintw(0,0,"Playing audio \n");
                        break;
			}
                case 'h':
                        {
			mvprintw(0,0,"Hold audio \n");
			alSourcePause(Source);
                        break;
                        }
                case 's':
                        {
                        mvprintw(0,0,"Stop audio \n");
			alSourceStop(Source);
                        break;
                        }
                default:
                        break;
             };
          };
        }
//END: EVENT-LOOP
//=================
//===========================
	alSourceStop(Source);
// CLOSE file
	fclose(fp);
// CLOSE AL
//retrive context and device
	ctx = alcGetCurrentContext();
	device = alcGetContextsDevice(ctx);
//close context and device	
	alcMakeContextCurrent(NULL);
	alcDestroyContext(ctx);
	alcCloseDevice(device);

// close ncurses window
	endwin();//ncurses closed	//*

//end
}
